import ctypes
import sys 
import os
import enum 

__all__         = ('MLOmgr',)
__version__     = "1.0.0"
__author__      = "MultilaneInc-ATE <support@multilaneinc.com>"
__date__        = '2021-09-16'

class  MLOType(enum.IntEnum):
    LR = 0,
    SR = 1,
    CWDM4 = 2,
    LR4 = 3,
    SR4 = 4,
    PSM4 = 5,
    SR86 = 6




DLLPATH32 = r"lib\x86\OpticalBox_API.dll"
DLLPATH64 = r"lib\x64\OpticalBox_API.dll"

class _IntPtr(ctypes.Structure):
    """Temporary structure for IntPtr creation."""
    pass

class IntPtr(ctypes.POINTER(_IntPtr)):
    """Structure pointer used as class pointer."""
    pass
class MLO(object):
    
    def __init__(self):
        self._Is64 = (sys.maxsize > 2**32)
        dll_path = os.path.join(os.path.dirname(os.path.realpath(__file__)),DLLPATH64 if self._Is64 else DLLPATH32)
        dlldir = os.path.dirname(dll_path)
        os.environ['PATH'] = dlldir + ";" 
        self._api = ctypes.cdll.LoadLibrary(dll_path)
        self._api.mlo_createInstance.restype = IntPtr
        self.instance = self._api.mlo_createInstance()
    
    def mlo_Connect(self,ip):
        IP = bytes(ip, encoding="ASCII")
        self._api.mlo_Connect.restype = ctypes.c_int
        return self._api.mlo_Connect(self.instance, IP)

    def mlo_Disconnect(self):
        self._api.mlo_Disconnect.restype = ctypes.c_int
        return self._api.mlo_Disconnect(self.instance)
    
    def mlo_ReadBoardID(self):
        self._api.mlo_ReadBoardID.restype = ctypes.c_int
        return self._api.mlo_ReadBoardID(self.instance)

    def mlo_ReadSerialNumber(self,SN, SerialNumberVersion):
        self._api.mlo_ReadSerialNumber.restype = ctypes.c_int
        return self._api.mlo_ReadSerialNumber(self.instance,SN,SerialNumberVersion)

    def mlo_ReadFirmwareRevision(self):
        self._api.mlo_ReadFirmwareRevision.restype = ctypes.c_double
        return self._api.mlo_ReadFirmwareRevision(self.instance)

    def mlo_TempRead(self, index):
        self._api.mlo_TempRead.restype = ctypes.c_double
        return self._api.mlo_TempRead(self.instance, ctypes.c_int(index))


    def mlo_SwitchSelect_2x2(self, index, data ):
        self._api.mlo_SwitchSelect_2x2.restype = ctypes.c_bool
        return self._api.mlo_SwitchSelect_2x2(self.instance, ctypes.c_ushort(index),ctypes.c_ushort(data))



    def mlo_SwitchSelect_MEMS_4x1(self, index, data ):
        self._api.mlo_SwitchSelect_MEMS_4x1.restype = ctypes.c_bool
        return self._api.mlo_SwitchSelect_MEMS_4x1(self.instance, ctypes.c_ushort(index),ctypes.c_ushort(data))
    
    def mlo_SwitchSelect_LBMM_4x1(self, index, data ):
        self._api.mlo_SwitchSelect_LBMM_4x1.restype = ctypes.c_bool
        return self._api.mlo_SwitchSelect_LBMM_4x1(self.instance, ctypes.c_ushort(index),ctypes.c_ushort(data))
    
    def mlo_ReadSwitchSelect_2x2(self, index, data ):
        self._api.mlo_ReadSwitchSelect_2x2.restype = ctypes.c_bool
        return self._api.mlo_ReadSwitchSelect_2x2(self.instance, ctypes.c_ushort(index),data)
   
    def mlo_ReadSwitchSelect_MEMS_4x1(self, index, data ):
        self._api.mlo_ReadSwitchSelect_MEMS_4x1.restype = ctypes.c_bool
        return self._api.mlo_ReadSwitchSelect_MEMS_4x1(self.instance, ctypes.c_ushort(index),data)
   
    def mlo_ReadSwitchSelect_LBMM_4x1(self, index, data ):
        self._api.mlo_ReadSwitchSelect_LBMM_4x1.restype = ctypes.c_bool
        return self._api.mlo_ReadSwitchSelect_LBMM_4x1(self.instance, ctypes.c_ushort(index),data)

    def mlo_ReadOpticalPower(self, index, data ):
        self._api.mlo_ReadOpticalPower.restype = ctypes.c_bool
        return self._api.mlo_ReadOpticalPower(self.instance, ctypes.c_ushort(index),data)

    def mlo_SetVOAAttenuation(self, index, data ):
        self._api.mlo_SetVOAAttenuation.restype = ctypes.c_bool
        return self._api.mlo_SetVOAAttenuation(self.instance, ctypes.c_ushort(index),ctypes.c_double(data))

    def mlo_ReadVOAAttenuation(self, index, data ):
        self._api.mlo_ReadVOAAttenuation.restype = ctypes.c_bool
        return self._api.mlo_ReadVOAAttenuation(self.instance, ctypes.c_ushort(index),data)

    def mlo_ReadTypeFromEEPROM(self,  Type ):
        self._api.mlo_ReadTypeFromEEPROM.restype = ctypes.c_bool
        return self._api.mlo_ReadTypeFromEEPROM(self.instance, Type)

    def mlo_ReadVOAInsertionLoss(self, index, Loss ):
        self._api.mlo_ReadVOAInsertionLoss.restype = ctypes.c_bool
        return self._api.mlo_ReadVOAInsertionLoss(self.instance, ctypes.c_ushort(index),Loss)

    def mlo_WriteVOAInsertionLoss(self, index, Loss ):
        self._api.mlo_WriteVOAInsertionLoss.restype = ctypes.c_bool
        return self._api.mlo_WriteVOAInsertionLoss(self.instance, ctypes.c_ushort(index),ctypes.c_double(Loss))


    def mlo_Read4x1SwitchInsertionLoss(self, index, Loss ):
        self._api.mlo_Read4x1SwitchInsertionLoss.restype = ctypes.c_bool
        return self._api.mlo_Read4x1SwitchInsertionLoss(self.instance, ctypes.c_ushort(index),Loss)

    def mlo_Write4x1SwitchInsertionLoss(self, index, Loss ):
        self._api.mlo_Write4x1SwitchInsertionLoss.restype = ctypes.c_bool
        return self._api.mlo_Write4x1SwitchInsertionLoss(self.instance, ctypes.c_ushort(index),ctypes.c_double(Loss))

    def mlo_ScreenControl(self, index, enable ):
        self._api.mlo_ScreenControl.restype = ctypes.c_bool
        return self._api.mlo_ScreenControl(self.instance, ctypes.c_ushort(index),ctypes.c_bool(enable))


    def mlo_SwitchSelect_MEMS_8x1(self, index, data ):
        self._api.mlo_SwitchSelect_MEMS_8x1.restype = ctypes.c_bool
        return self._api.mlo_SwitchSelect_MEMS_8x1(self.instance, ctypes.c_ushort(index), ctypes.c_ushort(data))


    def mlo_SwitchSelect_LBMM_8x1(self, index, data ):
        self._api.mlo_SwitchSelect_LBMM_8x1.restype = ctypes.c_bool
        return self._api.mlo_SwitchSelect_LBMM_8x1(self.instance, ctypes.c_ushort(index), ctypes.c_ushort(data))


    def mlo_ReadSwitchSelect_MEMS_8x1(self, index, data ):
        self._api.mlo_ReadSwitchSelect_MEMS_8x1.restype = ctypes.c_bool
        return self._api.mlo_ReadSwitchSelect_MEMS_8x1(self.instance, ctypes.c_ushort(index), data)

    def mlo_ReadSwitchSelect_LBMM_8x1(self, index, data ):
        self._api.mlo_ReadSwitchSelect_LBMM_8x1.restype = ctypes.c_bool
        return self._api.mlo_ReadSwitchSelect_LBMM_8x1(self.instance, ctypes.c_ushort(index), data)

